﻿using Proto;

namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 헤더 전파하기 - PropagateHeader(sender)

    /// <summary>
    /// 헤더 전파하기
    /// </summary>
    /// <param name="sender">발송자</param>
    /// <returns>발송자</returns>
    private static Sender PropagateHeader(Sender sender)
    {
        return (context, target, envelope) => sender(context, target, envelope.WithHeader(context.Headers));
    }

    #endregion
    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    private static void Main()
    {
        MessageHeader messageHeader = MessageHeader
            .Empty
            .With("UserID"  , "사용자1")
            .With("Language", "한국어");

        ActorSystem actorSystem = new ActorSystem
        (
            ActorSystemConfig.Setup() with
            {
                ConfigureRootContext = context => context.WithHeaders(messageHeader)
            }
        );

        Props props = Props.FromFunc
        (
            context =>
            {
                if(context.Message is CustomMessage message)
                {
                    Console.WriteLine($"메시지 : {message.Message}");

                    foreach((string key, string value) in context.Headers)
                    {
                        Console.WriteLine($"  키 : {key}, 값 : {value}");
                    }
                }

                return Task.CompletedTask;
            }
        );

        PID pid = actorSystem.Root.Spawn(props);

        IRootContext rootContext = actorSystem
            .Root
            .WithSenderMiddleware(PropagateHeader);

        rootContext.Send(pid, new CustomMessage("메시지1"));

        Console.WriteLine("프로그램을 종료하기 위해서 아무 키나 눌러 주시기 바랍니다.");

        Console.ReadKey(false);
    }

    #endregion
}